@@ -788,28 +788,6 @@ void ModuleEchoLink::allMsgsWritten(void)
 } /* allMsgsWritten */
 
 
-/*
- *----------------------------------------------------------------------------
- * Method:    reportState
- * Purpose:   Called by the core system when the state of the module should
- *    	      be reported.
- * Input:     None
- * Output:    None
- * Author:    Tobias Blomberg / SM0SVX
- * Created:   2005-02-12
- * Remarks:   FIXME: Is this function needed?? Status reports are handled in
- *    	      TCL only I think.
- * Bugs:      
- *----------------------------------------------------------------------------
- */
-void ModuleEchoLink::reportState(void)
-{
-  stringstream ss;
-  ss << "status_report " << numConnectedStations();
-  processEvent(ss.str());
-} /* reportState */
-
-
 /*
  *----------------------------------------------------------------------------
  * Method:    onStatusChanged
@@ -1344,8 +1322,8 @@ void ModuleEchoLink::createOutgoingConnection(const StationData &station)
 	processEvent(ss.str());
 	return;
       }
-      qsos.erase(it);
       qso = *it;
+      qsos.erase(it);
       qsos.push_back(qso);
       break;
     }
@@ -1844,6 +1822,7 @@ void ModuleEchoLink::handleCommand(const string& cmd)
     if (count > 0)
     {
       srand(time(NULL));
+        // coverity[dont_call]
       size_t random_idx = (size_t)(count * ((double)rand() / (1.0 + RAND_MAX)));
       StationData station = nodes[random_idx];
       
