@@ -41,6 +41,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include <utility>
 #include <list>
 #include <sigc++/bind.h>
+#include <sys/time.h>
 
 
 /****************************************************************************
@@ -53,6 +54,8 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include <AsyncAudioFifo.h>
 #include <AsyncAudioSelector.h>
 #include <AsyncAudioValve.h>
+#include <AsyncPty.h>
+#include <AsyncPtyStreamBuf.h>
 
 
 /****************************************************************************
@@ -613,25 +616,28 @@ void Voter::resetAll(void)
 
 void Voter::printSquelchState(void)
 {
+  stringstream os;
+  os << setfill('0') << std::internal;
+
   list<SatRx *>::iterator it;
   for (it=rxs.begin(); it!=rxs.end(); ++it)
   {
     float siglev = (*it)->signalStrength();
     bool sql_is_open = (*it)->squelchIsOpen();
 
-    cout << (*it)->name();
+    os << (*it)->name();
     if (sql_is_open)
     {
-      cout << ((*it) == sm->activeSrx() ? "*" : ":");
+      os << ((*it) == sm->activeSrx() ? "*" : ":");
     }
     else
     {
-      cout << " ";
+      os << "_";
     }
-    cout << left << setw(4) << static_cast<int>(siglev);
-    cout << " ";
+    os << showpos << setw(4) << static_cast<int>(siglev) << noshowpos;
+    os << " ";
   }
-  cout << endl;
+  publishStateEvent("Voter:sql_state", os.str());
 } /* Voter::printSquelchState */
 
 
@@ -971,6 +977,7 @@ int Voter::ActiveRxSelected::sqlRxId(void)
 
 void Voter::ActiveRxSelected::changeActiveSrx(SatRx *srx)
 {
+  voter().selector->selectSource(srx);
   activeSrx()->setMuteState(MUTE_CONTENT);
   box().active_srx = srx;
   if (muteState() == Rx::MUTE_NONE)
@@ -999,6 +1006,7 @@ void Voter::SquelchOpen::entry(void)
   
   runTask(bind(mem_fun(voter(), &Voter::setSquelchState), true));
   runTask(bind(mem_fun(activeSrx(), &SatRx::stopOutput), false));
+  runTask(mem_fun(voter(), &Voter::printSquelchState));
 } /* Voter::SquelchOpen::entry */
 
 
@@ -1019,6 +1027,7 @@ void Voter::SquelchOpen::exit(void)
   }
   
   runTask(bind(mem_fun(voter(), &Voter::setSquelchState), false));
+  runTask(mem_fun(voter(), &Voter::printSquelchState));
 } /* Voter::SquelchOpen::exit */
 
 
@@ -1117,7 +1126,7 @@ void Voter::Receiving::exit(void)
 
 void Voter::Receiving::timerExpired(void)
 {
-  //voter().printSquelchState();
+  voter().printSquelchState();
   
   assert(activeSrx() != 0);
   //assert(bestSrx() != 0);
